@@ -13,15 +13,15 @@
 //! ```
 //! let results_stream = Fetcher::default()
 //!     // Define a max number of ranged connections per file.
-//!     .connections_per_file(NonZeroU16::try_from(4).ok())
+//!     .connections_per_file(4)
 //!     // Max size of a connection's part, concatenated on completion.
-//!     .max_part_size(NonZeroU32::try_from(4 * 1024 * 1024).unwrap())
+//!     .max_part_size(4 * 1024 * 1024)
 //!     // An `Arc<AtomicBool>` which can be used to interrupt the download.
 //!     .cancel(cancellable)
 //!     // The channel for sending progress notifications.
 //!     .events(events_tx)
 //!     // Maximum number of retry attempts.
-//!     .retries(NonZeroU16::try_from(3).unwrap())
+//!     .retries(3)
 //!     // How long to wait before aborting a download that hasn't progressed.
 //!     .timeout(Duration::from_secs(15))
 //!     // Finalize the struct into an `Arc` for use with fetching.
@@ -267,6 +267,12 @@ impl<Data: Send + Sync + 'static> Fetcher<Data> {
                 .await
         };
 
+        let cleanup = || async {
+            remove_parts(&to).await;
+
+            self.send(|| (to.clone(), extra.clone(), FetchEvent::Fetched));
+        };
+
         let mut attempts = 0;
 
         let result = match fetch().await {
@@ -274,13 +280,17 @@ impl<Data: Send + Sync + 'static> Fetcher<Data> {
             Err(mut why) => {
                 while attempts < self.retries {
                     attempts += 1;
+
+                    if self.cancelled() {
+                        cleanup().await;
+                        return Err(Error::Cancelled);
+                    }
+
                     self.send(|| (to.clone(), extra.clone(), FetchEvent::Retrying));
 
                     match fetch().await {
                         Ok(()) => {
-                            remove_parts(&to).await;
-
-                            self.send(|| (to.clone(), extra.clone(), FetchEvent::Fetched));
+                            cleanup().await;
                             return Ok(());
                         }
 
@@ -292,9 +302,7 @@ impl<Data: Send + Sync + 'static> Fetcher<Data> {
             }
         };
 
-        remove_parts(&to).await;
-
-        self.send(|| (to.clone(), extra.clone(), FetchEvent::Fetched));
+        cleanup().await;
 
         result
     }
@@ -331,6 +339,7 @@ impl<Data: Send + Sync + 'static> Fetcher<Data> {
                                 self.send(|| (to, extra.clone(), FetchEvent::AlreadyFetched));
                                 return Ok(());
                             } else {
+                                error!("removing file with outdated timestamp: {:?}", to);
                                 let _ = fs::remove_file(to.as_ref())
                                     .await
                                     .map_err(Error::MetadataRemove)?;
